/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * CASTLE.H  Routines for interfacing with the Castle IDE
 */

#ifndef _CAS_CASTLE_H_
#define _CAS_CASTLE_H_

/* castle code type. should hold 0-32767 for compat. with Delphi Castle. */
typedef unsigned short int      castle_code;

#ifndef NO_CASTLE

#ifdef CAS

/* open and close communications channel */
void castle_open(char *argw);
void castle_close(void);

/* send formatted data */
void castle_raw(castle_code type, const char *fmt,...);
void castle_comment(const char *fmt,...);
void castle_format(castle_code base, formats format, char *filename);

/* base send command */
void castle_send(castle_code type, const char *fmt, va_list args);

#endif /* CAS */

/*
 * message codes.  Generally used as a base-10 bitfield
 */

#define CASTLE_UNKNOWN          0

/* bases for message `classes' */
#define CASTLE_GENERAL          0
#define CASTLE_FMT_INPUT        0x100
#define CASTLE_FMT_OUTPUT       0x200
#define CASTLE_FMT_LIST         0x300
#define CASTLE_TYPE             0x500

/* secondd tier */
#define CASTLE_TYPE_PROGRAM     0x10
#define CASTLE_TYPE_SCREENSHOT  0x20

/* information offsets */

#define CASTLE_COMMENT          0
#define CASTLE_OUTPUT           1

#define CASTLE_FMT_CTF          1
#define CASTLE_FMT_CAS          2
#define CASTLE_FMT_FXP          3
#define CASTLE_FMT_BMP          4
#define CASTLE_FMT_GIF          5
#define CASTLE_FMT_COM          6
#define CASTLE_FMT_BIN          7

#define CASTLE_TYPE_SINGLE      1
#define CASTLE_TYPE_MULTI       2
#define CASTLE_TYPE_SINGLEED    3
#define CASTLE_TYPE_MULTIED     4

#define CASTLE_TYPE_MONO        1
#define CASTLE_TYPE_COLOUR      2

/* other defines */

#define CASTLE_BUFSIZE          1024

#ifdef _CAS_CASTLE_C_
#endif /* _CAS_CASTLE_C_ */

#endif /* !NO_CASTLE */

#endif /* _CAS_CASTLE_H_ */

/* end of castle.h */
