/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * CONVERT.C  Type->Type conversions
 */

#define _CAS_CONVERT_C_

#include "cas.h"
#include "misc.h"
#include "args.h"
#include "token.h"
#include "list.h"
#include "convert.h"

/* larger conversion routines */

#include "vdrawss.h"

/*
 * Decides which conversions to perform where
 */

/* `after' must be exactly 1 or 0. */

void ConvertData(int after, casinfo *info)
{
  casinfo *curblk = NULL;
  convs *curconv = NULL;
  convprocs_t *curproc = NULL;
  int match;

  /*
   * Currently only looks for a direct conversion.  This driver routine needs
   * to be updated to be able to route conversions through others.  For all
   * normal conversions, all that is required is routing through one other
   * conversion, which shouldn't be difficult.  eg -c=SSMono-Oldprog-Editor
   */

  /* we loop through the info blocks first so that conversions happen once */

  for(curblk=info;curblk;curblk=curblk->next) {
    /* loop through specified conversions, searching for matching from */
    for(curconv=flags.convert;curconv;curconv=curconv->next) {
      /* check whether conversion should be done now */
      if (curconv->after==after && curconv->from==curblk->type) {
        Msg(M_DEBUG,"convert","trying to convert block of type %s",
                                    s_types[curblk->type]);
        match=FALSE;
        /* loop through available conversions, match from and to */
        for(curproc=convprocs;curproc->convertor;curproc++) {
          if (curproc->from==curconv->from && curproc->to==curconv->to) {
            /* match! -- specified and possible */
            match=TRUE;
            (*curproc->convertor)(curblk);
          }
        }
        if (match) {
          break;        /* breaks out of for(curconv...) loop */
        } else {
          Msg(M_WARN,"convert","conversion %s->%s unsupported",
                s_types[curconv->from],s_types[curconv->to]);
        }
      }
    }
  }
}

static void cv_SSMono_OldProg(casinfo *info)
{
  Msg(M_DEBUG,"convert","convert SS->PRG");
  DrawSSMono(info);
}

static void cv_OldProg_Editor(casinfo *info)
{
    Msg(M_DEBUG,"convert","convert PRG->ED");
}

static void cv_SSMono_SSCol(casinfo *info)
{
  Msg(M_DEBUG,"convert","convert SSMono->SSCol");

  info->type=tSSCol;    /* all that's required here! */

  /*
   * This conversion should also set the palette entry for colour 0
   * to black, instead of blue or something.  Palettes aren't implemented
   * yet however, so Mono screenshots converted to colour and saved as a
   * BMP or GIF will have *blue* text.  blue/black should really be an
   * option, as some mono calculators have blue text, and others have black
   */

}

/* end of convert.c */
