/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/* 
 * FBIN.C  Routines for reading and writing raw binary data
 */

#define _CAS_FBIN_C_

#include "cas.h"
#include "misc.h"
#include "args.h"
#include "list.h"
#include "fbin.h"

void BINWriteData(char *filename,casinfo *info)
{
  FILE *bin=NULL;

  if (!(bin=tsys_fopen(filename,"wb",FALSE))) {
    Msg(M_ERROR,"bin.write","Unable to open `%s'",filename);
    return;
  }

  for (;info;info=info->next) {
    if (info->type==tDesc) continue;
    if (info->type != tRaw) {
      Msg(M_WARN,"bin.write","implicit conversion of `%s' to raw data",
            s_types[info->type]);
    }
    if (info->length)
      fwrite(info->data,1,info->length,bin);
  }

  tsys_fclose(bin);
}


void BINReadData(char *filename, casinfo *info)
{
  FILE *bin=NULL;
  casinfo *curblk;
  unsigned char *data=NULL;
  long datalen=0;
  long memsize=BIN_STARTSIZE;
  int c;

  if (!(bin=tsys_fopen(filename,"rb",FALSE))) {
    Msg(M_ERROR,"bmp/read","Unable to open `%s'",filename);
    return;
  }

  curblk=tsys_malloc(sizeof(casinfo));
  curblk->type=tRaw;
  curblk->length=0;
  curblk->next=NULL;

  /*
   * To enable CaS to read binary data from a pipe, we can't use
   * fseek() and ftell() to work out the filesize, so the file is
   * read character by character.  In order to be able to read files
   * of `any' size, the memery pool is doubled when it's size is exceeded.
   * This behaviour is optimised for DJGPP's malloc, which rounds up to
   * the nearest power of 2 when allocating memory.  However, it also adds
   * an overhead to the amount requested, so we need to request slightly
   * less in order not to request f.e. 1.01Mb and be given 2Mb
   */

  data=tsys_malloc(memsize - BIN_OVERHEAD);

  for(;;) {
    c=getc(bin);
    if (feof(bin)) break;
    data[datalen++]=c;
    if (datalen>=memsize - BIN_OVERHEAD) {
      Msg(M_DEBUG,"bin.read","Reached threshold of 0x%lx bytes, increasing to 0x%lx",
        memsize - BIN_OVERHEAD,(memsize << 1) - BIN_OVERHEAD);
      memsize <<= 1;
      if (memsize - BIN_OVERHEAD > BIN_CEILING) {
        Msg(M_WARN,"bin.read","Reached memory ceiling of 0x%08x bytes, cropping.",
                                BIN_CEILING);
        break;
      }
      data=tsys_realloc(data,memsize - BIN_OVERHEAD);
    }
  }

  Msg(M_DEBUG,"bin.read","Data size: %ld bytes",datalen);

  tsys_fclose(bin);

  curblk->data=data;
  curblk->length=datalen;
  info->next=curblk;
}

/* end of fbin.c */
