/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * LIST.C  Routines for listing data to stdout
 */

#define _CAS_LIST_C_

#include "cas.h"
#include "misc.h"
#include "args.h"
#include "token.h"
#include "fcas.h"
#include "list.h"


void ListTypes(casinfo *info)
{
  Output("\n");
  for(;info;info=info->next) {
    switch (info->type) {
      case tDesc:
        ListDesc(info);
        break;
      case tOldProg:
        Output("%ld bytes \t",info->length);
        if (info->s.prg.prognum < 0 || info->s.prg.prognum > 37)
          Output("Program   ");
        else
          Output(" Program %s",ProgramName[info->s.prg.prognum]);
        if (info->s.prg.progtype != 0) {
          Output("\t(");
          if (info->s.prg.progtype & CAS_PT_SD)     Output(" Std. Dev.");
          if (info->s.prg.progtype & CAS_PT_LR)     Output(" Lin. Reg.");
          if (info->s.prg.progtype & CAS_PT_MATRIX) Output(" Matrix");
          if (info->s.prg.progtype & CAS_PT_BASEN)  Output(" Base-n");
          if (info->s.prg.progtype & CAS_PT_SSTO)   Output(" Store stats");
          if (info->s.prg.progtype & CAS_PT_SDRAW)  Output(" Draw data");
          Output(" )");
        }
        Output(".\n");
        break;

      case tEditor:
        Output("%ld bytes \t",info->length);
        Output("Program \"%s\"",code_to_text(info->s.ed.filename,TOKEN_LIST_STR));
        if (strlen(code_to_text(info->s.ed.password,TOKEN_LIST_STR)))
          Output("\t(Password protected)");
        Output(".\n");
        break;

      case tSSMono:
        Output("%d x %d \t",info->s.ss.width,info->s.ss.height);
        Output("Mono Screenshot.\n");
        break;
        
      case tSSCol:
        Output("%d x %d \t",info->s.ss.width,info->s.ss.height);
#ifdef PROPER_SPELLING
        Output("Colour Screenshot.\n");
#else
        Output("Color Screenshot.\n");
#endif
        break;

      case tRaw:
        Output("%4ld bytes \tRaw data",info->length);
        if (info->s.raw.cooked)
          Output(" (cooked)");
        Output("\n");
        break;

      default:
        Output("%4ld bytes \t%s\n",info->length,s_types[info->type]);
    }
  }
}

void ListDesc(casinfo *info)
{

}

void ListData(casinfo *info)
{
  darg_t dargs;

  args_join(flags.list.args,ARGS_DEF_LIST,0,&dargs);

  if (args_is_set(dargs,"dump",NULL)) {
    for(;info;info=info->next)
      ListDump(info);
    return;
  }

  for(;info;info=info->next) {
    switch (info->type) {
      case tSSMono:
      case tSSCol:
        ListScreenshot(info);
        break;
      case tOldProg:
      case tEditor:
        ListProgram(info);
        break;
      case tDesc:
      case tText:
        break;
      default:
        Msg(M_ERROR,"list","unsupported file type: %s",s_types[info->type]);
        break;
    }
  }
}

int flg_nice = 0;

void ListProgram(casinfo *info)
{
  darg_t dargs;
  int ctoken;
  unsigned char *data=NULL;
  char numstyle='0';

  args_join(flags.list.args,ARGS_DEF_LIST,info->type,&dargs);

  if (args_is_set(dargs,"space","spc",NULL)) numstyle='s';
  if (args_is_set_keyed(dargs,"num","oct","octal",NULL)) numstyle='o';
  if (args_is_set_keyed(dargs,"num","dec","decimal",NULL)) numstyle='d';
  if (args_is_set_keyed(dargs,"num","hex","hexadecimal",NULL)) numstyle='h';
  flg_nice = args_is_set(dargs,"nice",NULL);

  if (info->length < 2) return;

  Output("\n");
  if (info->type==tEditor) {
    Output("@@display program \"%s\"", code_to_text(info->s.ed.filename,TOKEN_LIST_STR));
    if (args_is_set(dargs,"password","pw",NULL) &&
        info->s.ed.password[0]) {
      Output(" (%s)", code_to_text(info->s.ed.password,TOKEN_LIST_STR));
    }
    Output("\n");
  } else {
    if (info->s.prg.prognum >= 0 &&
        info->s.prg.prognum <= 37) {
      Output("@@display program %s",ProgramName[info->s.prg.prognum]);
    } else {
      Output("@@display program");
    }
  }
  Output("\n");

  data=info->data;
  for (;;) {
    ctoken=*(data++);
    if (ctoken==0x7f) ctoken=256 + *(data++);
    if (ctoken==0xf7) ctoken=512 + *(data++);
    if (ctoken==0) {
      Msg(M_DEBUG,"list","Program terminated with 0");
      break;
    }
    if (ctoken==0xff) break;
    ListToken(ctoken,numstyle);
  }
  Output("\n");
  if (flags.pager) Pager(PAGER_ADD);
}


void ListToken(int ctoken, char numstyle)
{
  static char tok[TOKEN_MAX];

  strcpy(tok, code_to_token(ctoken, flg_nice?TOKEN_LIST_NICE:TOKEN_LIST_STR));

  switch (numstyle) {
    case 's': Output(" %s",tok);break;
    case 'd': Output(" {%d} %s",ctoken,tok);break;
    case 'h': Output(" {0x%03x} %s",ctoken,tok);break;
    case 'o': Output(" {0%03o} %s",ctoken,tok);break;
    default:
      Output("%s",tok);
  }
  fflush(stdout);
  if (flags.pager && strchr(tok,'\n')) Pager(PAGER_ADD);
}


void ListDump(casinfo *info)
{
  darg_t dargs;
  unsigned char *data=NULL;
  int p;

  args_join(flags.list.args,ARGS_DEF_LIST,0,&dargs);

  /* display common info in the (casinfo) block */

  Output("Info: %s, data=%p, length=%ld",s_types[info->type],
                                         info->data, info->length);

  /* Display type specific info */

  switch (info->type) {
    case tOldProg:
        Output(", progtype=%d, prognum=%d",info->s.prg.progtype,
                    info->s.prg.prognum);
        break;
    case tEditor:
        Output(", filename='%s', password='%s'",info->s.ed.filename,
                    info->s.ed.password);
        break;
    case tSSMono:
    case tSSCol:
        Output(", width=%d, height=%d, bpp=%d, order=%d",
                    info->s.ss.width,
                    info->s.ss.height,
                    info->s.ss.bpp,
                    info->s.ss.order);
        break;
    default:;
  }
  Output("\nData:\n");

  /* Display raw data.  This could also take type into account to perhaps
     make it look nicer */

  data=info->data;
  for (p=0;p<info->length;p++) {
    Output("%02x, ",*(data++));
  }
  /* Show next item of data *after* the real data, this is controlled by
     a switch as theoretically it could cause a Segmentation Violation */

  if (args_is_set(dargs,"shownext",NULL))
    Output("(%02x)",*data);       

  Output("\n\n");
}


void ListScreenshot(casinfo *info)
{
  darg_t dargs;
  int x,y;
  unsigned char *data;
  int top,bot;
  int flg_inv,flg_sng,flg_crop,flg_num;
  char topchar,botchar,fullchar,emptychar;

  args_join(flags.list.args,ARGS_DEF_LIST,info->type,&dargs);

  flg_inv=args_is_set(dargs,"inverse","inv",NULL);
  flg_crop=args_is_set(dargs,"crop",NULL);
  flg_sng=args_is_set(dargs,"single",NULL);
  flg_num=args_is_set(dargs,"num","linenos",NULL);

  topchar=SS_DEF_TOP;botchar=SS_DEF_BOTTOM;
  fullchar=SS_DEF_FULL;emptychar=SS_DEF_EMPTY;

  topchar=args_get_char(dargs,topchar,"top",NULL);
  botchar=args_get_char(dargs,botchar,"bottom","bot",NULL);
  fullchar=args_get_char(dargs,fullchar,"full",NULL);
  emptychar=args_get_char(dargs,emptychar,"empty","blank",NULL);

  data=info->data;

#if 0
  for (y=info->s.ss.height - 1;y>=0;y-=(flg_sng?1:2)) {
    if (flg_num) Output("%3d",y+1);
    for (x=0;x<(flg_crop?flg_num?76:79:info->s.ss.width);x++) {
      top=(*data)[x][y];
      if (flg_sng) bot=top; else bot=(*data)[x][y-1];
      if (flg_inv) { top=1-top; bot=1-bot; }
      Output("%c",top?bot?fullchar:topchar:bot?botchar:emptychar);
    }
    Output("\n");
    if (flags.pager) Pager(PAGER_ADD);
  }
#endif

  for(y=0;y<info->s.ss.height;y++) {
    for(x=0;x<79;x++) {
      switch (data[x+info->s.ss.width * y]) {
        case 0: Output("%c",' '); break;
        case 1: Output("%c",''); break;
        case 2: 
        case 3: Output("%c",''); break;
        case 4:
        case 5:
        case 6:
        case 7: Output("%c",''); break;
        default: Output("x");
      }
    }
    Output("\n");
  }
}

/* end of list.c */
