/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * MAIN.C  Parameter reading, and calling of input, output and list fn's
 */

/* 96/12/20
 * New standard terminology:
 *  Format   : CAS, TXT, FXP etc.
 *  Type     : program, screenshot, matrix etc.
 *  subformat: (subf) 9700 style CAS file vs 9850 style
 */

#define _CAS_MAIN_C_
#define CAS_DEFINE_GLOBALS

#include "cas.h"
#include "misc.h"
#include "args.h"
#include "token.h"
#include "list.h"
#include "fctf.h"
#include "fcas.h"
#include "ffxp.h"
#include "fbin.h"
#include "fbmp.h"
#include "fgif.h"
#include "fcom.h"
#include "convert.h"
#include "castle.h"
#include "main.h"

int main(int argc, char *argv[])
{
  char *argw=NULL;           /* writeable copy of *argv */
  char *fmt=NULL;            /* temporary string for format specifier */
  casinfo *base=NULL;        /* base for linked-list of block information */
  convs *conv_ptr=NULL;
  char *conv_from=NULL;
  char *conv_to=NULL;

  argv0 = *argv;

  if (argc == 1) {
    printf("%s\n", CAS_BANNER);
    return 0;
  }

  /*
   * Load default switch settings from .casrc.  As these are loaded before
   * command line processing (to enable macro expansion) we need to do a
   * specific check for the -d debug switch to allow debug messages during
   * args_load_defaults().  This is only supported as the first argument.
   */

  if (!strncmp(argv[1],"-d",2) || !strncmp(argv[1],"--debug",7)) {
    flags.debug=stderr;
    if ((argv[1])[1] == '-')
      argw = argv[1] + 7;
    else
      argw = argv[1] + 2;
    if (*argw == '=' || *argw == ':')
      argw++;
    if (*argw)
      flags.debug = tsys_fopen(argw,"w",FALSE);
    if (!flags.debug)
      Msg(EXIT_CMD,"main","unable to open debug file `%s'",argw);
    Msg(M_DEBUG,"main","debug messages active from very beginning");
  }
     
  args_load_defaults();

  /*
   * Load information from command line into flags structure
   */

  for (argv++, argc--; argc; argv++, argc--) {
    if ((**argv == '-') && (*argv)[1] != '\0') {

      if ((*argv)[1] == '-') {
        argw = shorten_long_opt(*argv);
      } else {
        argw = *argv;
      }
        
      switch (argw[1]) {

/* Display help info */

	    case 'V':
		  printf("%s\n", CAS_BANNER);
	      return 0;
        case '?': case 'h': 
          Output("%s\n", CAS_BANNER);
          Output("usage: CaS [-i[=fmt][,args]] file [-o[=fmt] outfile] [-l[=fmt]] [opt]\n\n");
          Output("infile     file to process\n");
          Output("outfile    output file to create\n");
          Output("-l[=format]  list file to stdout\n");
          Output("-i[=format]  force type and specify input file to type\n");
          Output("-o[=format]  select type of and specify output file\n");
          Output("format\n");
          Output("  ctf      calculator text format\n");
          Output("  dos      dos text (only recommended for with -l)\n");
          Output("  cas      casio program (.cas)\n");
          Output("  fxp      fxp program (.fxp)\n");
          Output("  bmp      windoze bitmap (.bmp)\n");
          Output("  gif      graphics interchange format (.gif)\n");
          Output("  com      serial communications to calculator\n");
          Output("options\n");
          Output("-v         verbose mode\n");
          Output("-p         pager\n");
          Output("-d[=file]  display debug information [in file]\n");
          Output("\nSee \"http://www.york.ac.uk/~tw104/\" for more info and updates.\n");

          return 0;

/* Set input filetype format */

        case 'i':   /* specifies input filetype and filename */
          if (flags.input.filename)
            Msg(EXIT_CMD,"main","Duplicate option: %s",argw);

          flags.input.args = args_split(argw+2,NULL);
          fmt = args_get_first(&flags.input.args);
          flags.input.format = string_to_format(fmt);

          argc--,argv++;
          if (!argc) Msg (EXIT_CMD,"main","input file not specified");
          flags.input.filename = *argv;
          break;
          
/* Set output filename and filetype (-oX filename) */

        case 'o':	/* specifies output filetype and filename */
          if (flags.output.filename)
            Msg(EXIT_CMD,"main","Duplicate option: %s",argw);

          flags.output.args = args_split(argw + 2,NULL);
          fmt = args_get_first(&flags.output.args);
          flags.output.format = string_to_format(fmt);

          argc--,argv++;
          if (!argc) Msg (EXIT_CMD,"main","output file not specified");
          flags.output.filename=*argv;
          break;
          
/* Set List output format */

        case 'l':   /* specifies list type */
          if ((flags.list.what & LIST_DATA) != 0)
            Msg(EXIT_CMD,"main","Duplicate option: %s",argw);
          flags.list.what |= LIST_DATA;
          flags.list.args = args_split(argw + 2, NULL);
          break;

/* Set Model */

        case 'm': 
          if (flags.model!=MDL_UNKNOWN) Msg(EXIT_CMD,"main","Duplicate option: %s",argw);

          argw+=2;if (*argw=='='||*argw==':') argw++;
          if (!strnicmp(argw,"cfx",3)) argw+=3;
          if (!strnicmp(argw,"fx",2)) argw+=2;

          if (!strcmp(argw,"7700") || !strcmp(argw,"7")) {
            flags.model=MDL_7700; break; }
          if (!strcmp(argw,"9700") || !strcmp(argw,"9")) {
            flags.model=MDL_9700; break; }
          if (!strcmp(argw,"9750") ) {
            flags.model=MDL_9750; break; }
          if (!strcmp(argw,"9800") || !strcmp(argw,"8")) {
            flags.model=MDL_9800; break; }
          if (!strcmp(argw,"9850") || !strcmp(argw,"5")) {
            flags.model=MDL_9850; break; }
          if (!strcmp(argw,"9950") ) {
            flags.model=MDL_9950; break; }
          if (!stricmp(argw,"any") || !strcmp(argw,"*")) {
            flags.model=MDL_UNKNOWN; break; }

          Msg(EXIT_CMD,"main","Unknown model: `%s'",argw);
          break;

/* Conversions */

        case 'C':
        case 'c': conv_ptr=tsys_malloc(sizeof conv_ptr);
                  conv_ptr->args=args_split(argw + 2,NULL);
                  conv_from=args_get_first(&conv_ptr->args);
                  conv_to=strchr(conv_from,'-');
                  if (!conv_to) {
                    Msg(M_ERROR,"main","Error in conversion specifier `%s'",conv_from);
                    free(conv_ptr->args); free(conv_ptr);
                    break;
                  }
                  *conv_to++='\0';
                  conv_ptr->from=string_to_type(conv_from);
                  if (conv_ptr->from==tUnknown) {
                    Msg(M_ERROR,"main","Unknown type specifier in conversion: %s",conv_from);
                    free(conv_ptr->args); free(conv_ptr);
                    break;
                  }
                  conv_ptr->to=string_to_type(conv_to);
                  if (conv_ptr->to==tUnknown) {
                    Msg(M_ERROR,"main","Unknown type specifier in conversion: %s",conv_to);
                    free(conv_ptr->args); free(conv_ptr);
                    break;
                  }
                  conv_ptr->after = (argw[1]=='C') ? 1 : 0;
                  conv_ptr->next=flags.convert;
                  flags.convert=conv_ptr;
                  break;


        case 't': flags.list.what |= LIST_TYPES;
                  break;

/* set miscellaneous options */

        case 'v': flags.verbose=TRUE;break;     /* Verbose mode */
        case 'd': if (flags.debug != NULL) break; /* ignore repeated -d */
        		  flags.debug=stderr;
                  argw+=2;if (*argw=='='||*argw==':') argw++;
                  if (*argw) flags.debug=tsys_fopen(argw,"w",FALSE);
                  if (!flags.debug)
                    Msg(EXIT_CMD,"main","unable to open debug file `%s'",argw);
                  Msg(M_DEBUG,"main","debug messages active from -d switch");
                  break;                        
        case 'e': castle_open(argw);        /* setup castle link */
                  castle_comment("%s\n",CAS_BANNER); break;
        case 'p': flags.pager=TRUE;break;       /* Screen Pager (more) */

        default : Msg(EXIT_CMD,"main","Unknown option: %s",*argv);
      }
    } else {

/* Set input filename */

      if (flags.input.filename && *flags.input.filename) {
        Msg(EXIT_CMD,"main","multiple input files not yet supported");
      } else {
        flags.input.filename=*argv;
      }
    }
  }
  
  if (flags.verbose) {
    Output("%s\n\n",CAS_BANNER);
  }
  
/* Check validity of switches */

  if (flags.input.filename == NULL) {
    Msg(EXIT_FILE,"main","No input file");
  }
  if (flags.output.filename == NULL && flags.list.what == 0) {
    flags.list.what |= LIST_TYPES;
  }
 
/* Heuristic to decide type of input file, if not over-ridden with -i */

  if (flags.input.format == fUnknown || flags.input.format == fDefault) {
    flags.input.format = format_from_filename(flags.input.filename);
    if (flags.input.format == fUnknown || flags.input.format == fDefault) {
      Msg(EXIT_FILE ,"main","undetermined input filetype");
    }
  }

/* Heuristic to decide type of output file if unspecified */

  if (flags.output.filename!=NULL && flags.output.format==fUnknown) {
    flags.output.format = format_from_filename(flags.output.filename);
    if (flags.output.format == fUnknown) {
      switch (flags.input.format) {
        case fCTF: 
        case fBMP:
        case fGIF: flags.output.format=fCAS;break;
        default:   flags.output.format=fDefault;
      }
      Msg(M_DEBUG,"main","output format: %s (deduced from input format: %s)",
        s_formats[flags.output.format],d_formats[flags.input.format]);
    }
  }

  Msg(M_DEBUG,"main","infile : %s (%s)",flags.input.filename,
                                        s_formats[flags.input.format]);
  Msg(M_DEBUG,"main","outfile: %s (%s)", flags.output.filename,
                                        s_formats[flags.output.format]);
  Msg(M_DEBUG,"main","list: (%d)", flags.list.what);

  /**********************************************************************
   *  Now act on the data in the flags                                  *
   **********************************************************************/

  /* Load tokens into memory */
  {
    darg_t in_args, list_args, out_args;
    char *in_tokf="", *out_tokf="", *list_tokf="";
    
    /* see if a token file is specified */
    args_join(flags.input.args, ARGS_DEF_IN, fCTF, &in_args);
    args_join(flags.output.args, ARGS_DEF_OUT, fCTF, &out_args);
    args_join(flags.list.args, ARGS_DEF_LIST, tOldProg, &list_args);
    in_tokf = args_get_string(in_args, "tokfile", NULL);
    out_tokf = args_get_string(out_args, "tokfile", NULL);
    list_tokf = args_get_string(list_args, "tokfile", NULL);

    /* if not, force it to the default if tokens will be required */

    if (flags.input.format == fCTF && *in_tokf == '\0') {
      in_tokf = CAS_DEFAULT_TOKFILE;
    }
    if ( (flags.output.format == fCTF || flags.output.format == fDefault)
         && *out_tokf == '\0') {
      out_tokf = CAS_DEFAULT_TOKFILE;
    }
    if (flags.list.what != 0 && *list_tokf == '\0') {
      list_tokf = CAS_DEFAULT_TOKFILE;
    } 
    
    Msg(M_DEBUG,"token","input tokfile:  `%s'",in_tokf);
    Msg(M_DEBUG,"token","output tokfile: `%s'",out_tokf);
    Msg(M_DEBUG,"token","list tokfile:   `%s'",list_tokf);
    
    token_init(in_tokf, out_tokf, list_tokf);
  }

/* Initialise memory */

  base = tsys_malloc(sizeof (casinfo));
  base->type = tDesc;
  base->data = NULL;
  base->length = 0;
  base->next = NULL;

/* Read data from input file */

  castle_format(CASTLE_FMT_INPUT,flags.input.format,flags.input.filename);
  switch (flags.input.format) { 
    case fCTF: ctf_read_data(flags.input.filename,base);break;
    case fCAS: CASReadData(flags.input.filename,base);break;
    case fFXP: FXPReadData(flags.input.filename,base);break;
    case fBIN: BINReadData(flags.input.filename,base);break;
    case fBMP: BMPReadData(flags.input.filename,base);break;
    /* GIF reading not yet supported */
    case fCOM: COMReadData(flags.input.filename,base);break;
    default  : Msg(EXIT_BUG,"main","input format not yet supported");
  } 

/* Perform conversions (before) */

  ConvertBefore(base);

/* List types summary */

  if ((flags.list.what & LIST_TYPES) != 0)
    ListTypes(base);

/* List info */

/*  castle_format(CASTLE_FMT_LIST,flags.list.format,"stdout"); */

  if ((flags.list.what & LIST_DATA) != 0) {
    ListData(base);
  }

/* Perform conversions (after) */

  ConvertAfter(base);

/* if default output format, decide now what it should be */

  if (flags.output.format == fDefault) {
    switch (base->type) {
      case tRaw:        flags.output.format = fBIN;
                        break;
      case tSSMono:
      case tSSCol:      flags.output.format = fBMP;
                        break;
      default:          flags.output.format = fCTF;
    }
    Msg(M_DEBUG,"main","output file format: %s (deduced from input type: %s)",
      s_formats[flags.output.format], s_types[base->type]);
  }

/* Write data to output file */

  if (flags.output.filename) {
    castle_format(CASTLE_FMT_OUTPUT,flags.output.format,flags.output.filename);
    switch (flags.output.format) {
      case fCTF: ctf_write_data(flags.output.filename,base);break;
      case fFXP: FXPWriteData(flags.output.filename,base);break;
      case fBIN: BINWriteData(flags.output.filename,base);break;
      case fBMP: BMPWriteData(flags.output.filename,base);break;
      case fGIF: GIFWriteData(flags.output.filename,base);break;
      case fCAS: CASWriteData(flags.output.filename,base);break;
      case fCOM: COMWriteData(flags.output.filename,base);break;
      default  : Msg(EXIT_BUG,"main","Output format not yet supported");
    }
  }

/* Tidy up */

  token_free();

  tsys_fclose(flags.debug);
  castle_close();

  return 0; 
}


/*
 * format_from_filename.  Guess the file type of an input file.
 */


formats format_from_filename(char *filename)
{
  char *extn=NULL;

  Msg(M_DEBUG,"format","checking `%s'",filename);

  /* check for serial devices */

#ifdef CAS_DOS
  if (!strnicmp(filename,"COM",3) && !filename[4] &&
                                 (filename[3]>'0' && filename[3]<'5')) {
    Msg(M_DEBUG,"filetype","serial device (COMx)");
    return fCOM;
  }
#endif /* CAS_DOS */

#ifdef CAS_UNIX
  if (!strnicmp(filename,"/dev/tty",8)) {
    Msg(M_DEBUG,"filetype","serial device (/dev/ttyX)");
    return fCOM;
  }
#endif /* CAS_UNIX */

  /* check stdin, stdout, stderr */
  
  if (!strcmp(filename,"-") ||
      !strcmp(filename,"stdin") ||
      !strcmp(filename,"stdout") ||
      !strcmp(filename,"stderr")) {
    Msg(M_DEBUG,"filetype","standard file stream => ctf");
    return fCTF;
  }
  
  /* check extensions */
  
  extn = strrchr(filename,'.');
  if (extn && *extn) {
    extn++;
    if (!stricmp(extn,"cas")) {
      Msg(M_DEBUG,"filetype","CAS file (.cas)");
      return fCAS;
    }
    if (!stricmp(extn,"ctf") || !stricmp(extn,"txt")) {
      Msg(M_DEBUG,"filetype","Calculator Text File (.%s)",extn);
      return fCTF;
    }
    if (!stricmp(extn,"fxp")) {
      Msg(M_DEBUG,"filetype","FXP file (.fxp)");
      return fFXP;
    }
    if (!stricmp(extn,"bmp")) {
      Msg(M_DEBUG,"filetype","BMP file (.bmp)");
      return fBMP;
    }
    if (!stricmp(extn,"gif")) {
      Msg(M_DEBUG,"filetype","GIF file (.gif)");
      return fGIF;
    }
  }

  Msg(M_DEBUG,"filetype","unrecognised - setting to default");
  return fDefault;
}


/*
 * shorten_long_opt.
 * simple minded version of getopt_long which simply converts a long option
 * stored in the long_options array into the appropriate short option by
 * editing the actual parameter and returning a pointer to the new start
 */


char *shorten_long_opt(char *lo)
{
  int i;
  char *ptr;
  
  /* remember that the long_opt still begins with -- */
  for (i = 0; long_options[i].lo != NULL; i++) {
    if (!strncmp(lo + 2, long_options[i].lo, strlen(long_options[i].lo))) {
      ptr = lo + strlen(long_options[i].lo);
      ptr[0] = '-';
      ptr[1] = long_options[i].so;
      return ptr;
    }
  }
  
  return lo;
}


/* end of main.c */
