/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * MISC.C  Miscellaneous utility functions and std library wrappers
 */

#define _CAS_MISC_C_

#include "cas.h"
#include "castle.h"
#include "misc.h"

void Output (const char *fmt, ...)
{
  va_list args;

  va_start (args,fmt);
  vprintf(fmt,args);
  castle_send(CASTLE_OUTPUT,fmt,args);
  va_end (args);
}

void Msg (int type, const char *section, const char *msgstr, ...)
{
  FILE *outfile=stdout;
  char *fatal_msg[]= { "EXIT_BUG","EXIT_CMD","EXIT_FILE","EXIT_MEM",
  					   "EXIT_COMM","EXIT_FORMAT","EXIT_MAJOR_BUG" };

  va_list args;
  
  if (flags.debug) {
    switch (type) {
      case M_MSG:     fprintf(flags.debug,"message: ");break;
      case M_VERBOSE: fprintf(flags.debug,"verbose: ");break;
      case M_WARN:    fprintf(flags.debug,"warning: ");break;
      case M_ERROR:   fprintf(flags.debug,"error  : ");break;
      case M_DEBUG:   fprintf(flags.debug,"debug  : ");break;
      default:        fprintf(flags.debug,"fatal  :%s: ",fatal_msg[type]);
    }
    fprintf(flags.debug,"%s: ",section);
    va_start (args,msgstr);
    vfprintf(flags.debug,msgstr,args);
    va_end (args);
    putc('\n',flags.debug);
    if (type==M_DEBUG) return;
  }
  
  switch (type) {
    case M_MSG: break;
    case M_VERBOSE: if (!flags.verbose) return; break;
    case M_WARN:  outfile=stderr; fprintf(outfile,"warning: ");break;
    case M_ERROR: outfile=stderr; fprintf(outfile,"error: ");break;
    case M_DEBUG: return;
    default:      outfile=stderr; fprintf(outfile,"fatal:%s: ",fatal_msg[type]);
  }
  fprintf(outfile,"%s: ",section);
  va_start (args,msgstr);
  vfprintf(outfile,msgstr,args);
  va_end (args);
  putc('\n',outfile);

  if (outfile==stdout) Pager(PAGER_ADD);

  if (type>0) exit(type);
  if (type==0) exit(255);	/* bug */
}

/* pager.  broken */

void Pager(int dowhat)
{
  static int count=0;
  char buf[99];
  
/*  return; */
  
  if (dowhat==PAGER_CLEAR) {
    count=0;
    return;
  }

  if (!flags.pager && (!(dowhat&PAGER_FORCE)))
    return;
  
  if (!(dowhat&PAGER_NOADD)) ++count;

  if (count>=PAGER_LINES||(dowhat&PAGER_NOW)) {
    if (!(dowhat&PAGER_NORESET)) count=0;
    if (!(dowhat&PAGER_NOMSG))
      fprintf(stderr,"pager: press <enter> to continue. ");
    if (!(dowhat&PAGER_NOPAUSE)) fgets(buf,99,stdin);
  }
}

formats string_to_format(char *str)
{
  register int i;
  
  if (str == NULL)
    return fUnknown;
    
  for(i=0;i<CAS_NUM_FORMATS;++i) {
    if (!stricmp(str, s_formats[i])) return i;
  }
  return fUnknown;
}

types string_to_type(char *str)
{
  register int i;
  
  if (str == NULL)
    return tUnknown;  
    
  for(i=0;i<CAS_NUM_TYPES;++i) {
    if (!stricmp(str, s_types[i])) return i;
  }
  return tUnknown;
}


/* end of misc.c */